// $Id: nasdsim_component_client.C,v 1.21 2000/05/09 01:05:55 bforney Exp $

#include<strstream>
#include "nasdsim_debug.h"
#include "nasdsim_component_client.h"
#include "nasdsim_event_WRITE.h"
#include "nasdsim_event_READ.h"

component_client::component_client (sim_component_id_t eId, 
				    hash_map <sim_component_id_t, int> &maxCreditHash,
				    char *fileName,
				    sim_bandwidth_t lBW,
				    sim_len_t hdrSz,
				    bool dumpHist,
				    bool dumpInt)
{
  // KLUDGE: to get information on a client --------------------------------------
  
  dumpHistory = dumpHist;
  
  if (dumpHistory) {
    ostrstream o;
    o << "client_" << eId << "_history.txt";
    history_dump_file_ = new ofstream (o.rdbuf()->str ());
    if (!(*history_dump_file_)) { delete history_dump_file_; throw; }
  }
  
  dumpIntervals = dumpInt;
  
  if (dumpIntervals) {
    ostrstream o;
    o << "client_" << eId << "_interval.txt";
    interval_dump_file_ = new ofstream (o.rdbuf ()->str ());
    if (!(*interval_dump_file_)) { delete interval_dump_file_; throw; }
  }
  //------------------------------------------------------------------------------
  
  traceReqCnt = 0;
  entityId    = eId;
  link        = new component_link (eId, lBW, hdrSz);
  trace       = new tracefile (fileName, eId);
  
  ASSERT (maxCreditHash.size () > 0);
  
  // For each nasd
  //     1. make an entry into the CreditHash Table
  //     2. Create a queue of pending requests
  //     3. create a Stats object to keep track of performance data
  //     4. create a NasdStatus object to keep track of Interval info
  hash_map <sim_component_id_t, int>::iterator mCIter;
  
  for (mCIter = maxCreditHash.begin (); 
       mCIter != maxCreditHash.end (); 
       mCIter++) 
    {
      // Allocate CreditStatus object
      CreditStatus *cS = new CreditStatus;
      ASSERT (cS != NULL);
      
      cS->total = mCIter->second;
      cS->inUse = 0;
      creditHash [mCIter->first] = cS;
      
      // Allocate a queue
      queue <event *> * q = new queue <event *>;
      ASSERT (q != NULL);
      pQsHash [mCIter->first] = q;
      
      // Allocate a Stats object
      Stats *s = new Stats ();
      ASSERT (s != NULL);
      statsHash [mCIter->first] = s;
      
      NasdStatus *ns = new NasdStatus (mCIter->first);
      ASSERT (ns != NULL);
      nasdStatusHash [mCIter->first] = ns;
    }
}

component_client::~component_client()
{
  if (dumpHistory) delete history_dump_file_;
  
  if (dumpIntervals) delete interval_dump_file_;
  
  delete trace;
  
  delete link;
    
  for (PartialReadHashIter pHashIter = partialReadHash.begin (); 
	 pHashIter != partialReadHash.end (); 
       pHashIter++)
      if (pHashIter->second != NULL) delete pHashIter->second;
  
  for (CreditHashIter cHashIter = creditHash.begin (); 
       cHashIter != creditHash.end (); 
       cHashIter++)
      if (cHashIter->second != NULL) delete cHashIter->second;
  
  for (StatsHashIter sHashIter = statsHash.begin (); 
       sHashIter != statsHash.end (); 
       sHashIter++)
      if (sHashIter->second != NULL) delete sHashIter->second;
  
  for (PQsHashIter pHashIter = pQsHash.begin (); 
       pHashIter != pQsHash.end (); 
       pHashIter++)
      if (pHashIter->second != NULL) {
	  // delete queue
	  queue <event *> * q = pHashIter->second;
	  for (; !q->empty (); q->pop ()) if (q->front () != NULL) delete q->front ();
	  delete q;
      }
  
  for (StreamHashIter sHashIter = streamHash.begin (); 
       sHashIter != streamHash.end (); 
       sHashIter++)
      if (sHashIter->second != NULL) delete sHashIter->second;
  
  for (NasdStatusHashIter nHashIter = nasdStatusHash.begin (); 
       nHashIter != nasdStatusHash.end (); 
       nHashIter++)
    if (nHashIter->second != NULL) delete nHashIter->second;
}

/**
 * Increments the credit this client has for the given nasd
 */
void
component_client::incCredit (sim_component_id_t nId)
{
    ASSERT (creditHash.find (nId) != creditHash.end());
    CreditStatus *c = creditHash [nId];
    ASSERT (c != NULL);
    c->inUse--;
}


#if 0
void
component_client::updateStream (sim_stream_id_t streamId, sim_component_id_t nasdId)
{
    if (streamId == -1) return;
    
    ASSERT (streamHash.find (streamId) != streamHash.end());
    
    StreamStatus *s = streamHash [streamId];
    
    ASSERT (s != NULL);
    
    if (s->qEmpty ()) { s->setActive (false); return; }
    
    event * evt = s->dequeue ();
    
    ASSERT (pQsHash.find (nasdId) != pQsHash.end ());
    
    queue <event *> *q = pQsHash [nasdId];
    
    ASSERT (q != NULL);
    
    // Added (05/07/2000) -----------
    if (streamId == 69 && (this->entityId == 0)) {
      cout << "event [startTime = " << evt->get_start_time () 
	   << ", nasdId_given = " << nasdId << ", streamId_given = " << streamId 
	   << endl;
      sim_component_id_t nasd;
      sim_stream_id_t stream;
      
      if (dynamic_cast <event_READ_client_read *> (evt) != NULL) {
	nasd = (dynamic_cast <event_READ_client_read *> (evt))->get_nasd_id ();
	stream = (dynamic_cast <event_READ_client_read *> (evt))->get_stream_id ();
      }
      
      else if (dynamic_cast <event_WRITE_client_write *> (evt) != NULL) {
	  nasd = (dynamic_cast <event_WRITE_client_write *> (evt))->get_nasd_id ();
	  stream = (dynamic_cast <event_WRITE_client_write *> (evt))->get_stream_id ();
      }
      
      cout << "Suppose to be nasdId = " << nasd << ", stream = " << stream 
	   << endl;
    }
    
    // ----------------------------------------------------------------
    
    q->push (evt);
}

#endif

/**
 * Updates status of stream. Corresponding StreamStatus object containing pending 
 * requests is found. Event at front of StreamStatus is removed and added to
 * appropriate nasd queue of pending requests.
 *
 * streamId - id of stream
 * nasdId - id of nasd
 * returns - none
 */
void
component_client::updateStream (sim_stream_id_t streamId)
{
  if (streamId == -1) return;
  
  ASSERT (streamHash.find (streamId) != streamHash.end());
  
  StreamStatus *s = streamHash [streamId];
  
  ASSERT (s != NULL);
    
  if (s->qEmpty ()) { s->setActive (false); return; }
  
  event * evt = s->dequeue ();
  
  sim_component_id_t nasdId;
  
  if (dynamic_cast <event_READ_client_read *> (evt) != NULL)
    nasdId = (dynamic_cast <event_READ_client_read *> (evt))->get_nasd_id ();
  else if (dynamic_cast <event_WRITE_client_write *> (evt) != NULL)
    nasdId = (dynamic_cast <event_WRITE_client_write *> (evt))->get_nasd_id ();
  
  ASSERT (pQsHash.find (nasdId) != pQsHash.end ());
  
  queue <event *> *q = pQsHash [nasdId];
  
  ASSERT (q != NULL);
  
  q->push (evt);
}

/**
 * Takes event and determines if it should be queued or not
 *
 * evt - pointer to event
 * streamId - id of stream
 * returns - 'true' if event is not queued 'false' otherwise
 */
bool
component_client::isStreamAllowed (event *evt, sim_stream_id_t streamId) 
{
    if (streamId == -1) return true;
    
    // get queue for the streamId
    if (streamHash.find (streamId) != streamHash.end ()) {
	StreamStatus *s  = streamHash [streamId];
	ASSERT (s != NULL);
	
	if (s->getActive ()) {
	    s->enqueue (evt);
	}
	else {
	    s->setActive (true);
	    return true;
	}
    }
    
    // Make queue for streamId and place in streamHash
    else {
	StreamStatus *s = new StreamStatus ();
	ASSERT (s != NULL);
	s->setActive (true);
	streamHash [streamId] = s;
	return true;
    }
    
    return false;
}

/**
 * This method returns a list of events that are ready to go at this instant
 */
list <event *> *
component_client::getEventList (simtime_t tmpTime, simtime_t currTime)
{
  list <event *> *eventList;
  
  eventList = new list <event *>;
  
  checkNasdQueues (eventList, currTime);
  
  // Find all events from the tracefile whose start times are less than
  // or equal to the current time.
  
  // The events that qualify the above criterion but cannot be run becoz of lack
  // of credits are put on to a queue that is maintained for each nasd
  
  while (!trace->empty () && trace->front ()->get_start_time () <= tmpTime) {
      
      event * evt = const_cast<event *> (trace->front ());
      trace->pop ();
      
      // Added (05/07/00)--------------
      traceReqCnt++;
      //-------------------------------

      sim_component_id_t nasdId;
      sim_stream_id_t streamId;
      
      if (dynamic_cast <event_READ_client_read *> (evt) != NULL) {
	nasdId = (dynamic_cast <event_READ_client_read *> (evt))->get_nasd_id ();
	streamId = (dynamic_cast <event_READ_client_read *> (evt))->get_stream_id ();
      }
      
      else if (dynamic_cast <event_WRITE_client_write *> (evt) != NULL) {
	  nasdId = (dynamic_cast <event_WRITE_client_write *> (evt))->get_nasd_id ();
	  streamId = (dynamic_cast <event_WRITE_client_write *> (evt))->get_stream_id ();
      }
      
      else 
	ASSERT (0);
      
      if (!isStreamAllowed (evt, streamId)) continue;

      ASSERT (creditHash.find (nasdId) != creditHash.end ());
      CreditStatus * cS = creditHash [nasdId];
      ASSERT (cS != NULL);
      int avail = cS->total - cS->inUse;
      
      // If available, then add to eventList
      if (avail > 0) {
	  ASSERT (evt->get_start_time() >= evt->get_injection_time());
	  ASSERT (evt->get_injection_time () >= currTime);              /**/
	  eventList->push_front (evt);
	  cS->inUse++;
      }
      
      // Add evt to pending queue for this nasd
      else {
	  ASSERT (pQsHash.find (nasdId) != pQsHash.end ());
	  queue <event *> *q = pQsHash [nasdId];
	  q->push (evt);
      }
  }    
  
  return eventList;
}

#if 0
bool
component_client::hasMoreEvents(void)
{
  // first, check the client's pending event queues
  for (PQsHashIter pQsHashIter = pQsHash.begin (); 
       pQsHashIter != pQsHash.end (); 
       pQsHashIter++) {
    if (!pQsHashIter->second->empty()) {
      return true;
    }
  }
  
  // second, check each stream's pending event queues
  for (StreamHashIter sHashIter = streamHash.begin ();
       sHashIter != streamHash.end ();
       sHashIter ++)
      if (!sHashIter->second->qEmpty ()) return true;
  
  // third, check the tracefile
  return !trace->empty();
}
#endif

// Modifying this method (05/07/00)
bool
component_client::hasMoreEvents(void)
{
  // first, check the client's pending event queues
  for (PQsHashIter pQsHashIter = pQsHash.begin (); 
       pQsHashIter != pQsHash.end (); 
       pQsHashIter++) {
    
    sim_component_id_t nasdId = pQsHashIter->first;

    ASSERT (creditHash.find (nasdId) != creditHash.end ());
    
    CreditStatus * cS = creditHash [nasdId];
    
    ASSERT (cS != NULL);
    
    int avail = cS->total - cS->inUse;
    
    if (!pQsHashIter->second->empty() && avail > 0) {
      return true;
    }
  }
  
  // second, check the tracefile. We are using a PESSIMISTIC scheme here. If the event
  // at the front of the tracefile cannot be issued at this instant then we
  // return false
  if (trace->empty ()) return false;
  
  event *            evt;
  sim_component_id_t nasdId;
  sim_stream_id_t    streamId;
  
  evt = const_cast <event *> (trace->front ());
  
  if (dynamic_cast <event_READ_client_read *> (evt) != NULL) {
    nasdId = (dynamic_cast <event_READ_client_read *> (evt))->get_nasd_id ();
    streamId = (dynamic_cast <event_READ_client_read *> (evt))->get_stream_id ();
  }
  else if (dynamic_cast <event_WRITE_client_write *> (evt) != NULL) {
    nasdId = (dynamic_cast <event_WRITE_client_write *> (evt))->get_nasd_id ();
    streamId = (dynamic_cast <event_WRITE_client_write *> (evt))->get_stream_id ();
  }
  
  ASSERT (creditHash.find (nasdId) != creditHash.end ());
  
  CreditStatus * cS = creditHash [nasdId];
  
  ASSERT (cS != NULL);
  
  int avail = cS->total - cS->inUse;
  
  if (streamHash.find (streamId) != streamHash.end ()) {
    StreamStatus *s  = streamHash [streamId];
    ASSERT (s != NULL);
    if (!s->getActive () && avail > 0) return true;
    else return false;
  }
  
  return avail > 0;
}

void 
component_client::updateWriteStats (simtime_t startTime, 
				    simtime_t finishTime, 
				    sim_component_id_t nasdId, 
				    sim_len_t dataSize)
{
  double currBw = ((double)dataSize) / (finishTime - startTime);
  
  // Update status for each nasd (05/07/00)
  ASSERT (nasdStatusHash.find (nasdId) != nasdStatusHash.end ());
  NasdStatus *status = nasdStatusHash [nasdId];
  ASSERT (status != NULL);
  
  status->addBytes (dataSize);
  status->setLastBandwidth (currBw);
  //--------------------------------------------------------------
  
  dumpHistoryInfo (finishTime);
#if 0
  if (nasdId == 0) 
    {
      *dump_file_ << finishTime << " " << currBw << " " << prate1 << endl;
      prate0 = currBw;
    }
  else if (nasdId == 1) 
    {
      *dump_file_ << finishTime << " " << prate0 << " " << currBw << endl;
      prate1 = currBw;
    }
#endif
  //-------------------------------------------------------------------------------

  // Update stats for client
  clientStats.updateWrite (startTime, finishTime, dataSize);
  
  // Update stats for each nasd
  ASSERT (statsHash.find (nasdId) != statsHash.end ());
  Stats *s = statsHash [nasdId];
  ASSERT (s != NULL);
  s->updateWrite (startTime, finishTime, dataSize);
}

void
component_client::dumpIntervalInfo (simtime_t currTime)
{
  if (!dumpIntervals) return;
  
  *interval_dump_file_ << currTime;
  
  int n = nasdStatusHash.size ();
  
  for (int i = 0; i < n; i++) {
    ASSERT (nasdStatusHash.find (i) != nasdStatusHash.end());
    NasdStatus *nStatus = nasdStatusHash [i];
    ASSERT (nStatus != NULL);
    *interval_dump_file_ << " " << nStatus->getIntervalBandwidth (currTime); 
    nStatus->newInterval (currTime);
  }
  
  *interval_dump_file_ << endl;
}

void
component_client::dumpHistoryInfo (simtime_t currTime)
{
  if (!dumpHistory) return;
  
  *history_dump_file_ << currTime;
  
  int n = nasdStatusHash.size ();
  
  for (int i = 0; i < n; i++) {
    ASSERT (nasdStatusHash.find (i) != nasdStatusHash.end());
    NasdStatus *nStatus = nasdStatusHash [i];
    ASSERT (nStatus != NULL);
    *history_dump_file_ << " " << nStatus->getLastBandwidth (); 
  }
  
  *history_dump_file_ << endl;
}

void 
component_client::updateReadStats (simtime_t startTime, 
				   simtime_t finishTime, 
				   sim_component_id_t nasdId, 
				   sim_len_t dataSize)
{
  double currBw = ((double)dataSize) / (finishTime - startTime);
  
  // Update status for each nasd (05/07/00)
  ASSERT (nasdStatusHash.find (nasdId) != nasdStatusHash.end ());
  NasdStatus *status = nasdStatusHash [nasdId];
  ASSERT (status != NULL);
  
  status->addBytes (dataSize);
  status->setLastBandwidth (currBw);
  
  dumpHistoryInfo (finishTime);
  
#if 0
  //--------------------  
  if (nasdId == 0) 
    {
      *dump_file_ << finishTime << " " << currBw << " " << prate1 << endl;
      prate0 = currBw;
    }
  else if (nasdId == 1) 
    {
      *dump_file_ << finishTime << " " << prate0 << " " << currBw << endl;
      prate1 = currBw;
    }
  //-------------------------------------------------------------------------------
#endif
  
 // Update stats for client
  clientStats.updateRead (startTime, finishTime, dataSize);
  
  // Update stats for each nasd
  ASSERT (statsHash.find (nasdId) != statsHash.end ());
  Stats *s = statsHash [nasdId];
  ASSERT (s != NULL);
  s->updateRead (startTime, finishTime, dataSize);
}

void 
component_client::dump_stats (ostream & output)
{
  StatsHashIter sHashIter;
  
  output << "**************** Client Info ************************" << endl
	 << "CLIENT ID = " << entityId << endl
	 << "# of trace file requests read = " << traceReqCnt << endl;
  
  // (05/07/00) Added some debug code ----------------------------------------
  // check pending queue for each nasd
  ASSERT (partialReadHash.size () == 0);
  
  for (PQsHashIter pQsHashIter = pQsHash.begin (); 
       pQsHashIter != pQsHash.end (); 
       pQsHashIter++)
    ASSERT (pQsHashIter->second->size () == 0);
  
  // check pending queue for each stream
  for (StreamHashIter strHashIter = streamHash.begin ();
       strHashIter != streamHash.end ();
       strHashIter ++)
    ASSERT (strHashIter->second->size () == 0); 
  
  output << "Check: All pending queues are empty" << endl << endl;
  //---------------------------------------------------------------------------
  
  clientStats.dumpStats (output);
  
  for (sHashIter = statsHash.begin (); 
       sHashIter != statsHash.end (); 
       sHashIter++) {
    output << "NASD ID = " << sHashIter->first << endl;
    sHashIter->second->dumpStats (output);
  }
  
  output << "************** End Client Info **********************" << endl;
}

/**
 * Check pending event queue of each nasd. Dequeue and tack onto eventList as many
 * events as is permitted by credits for that nasd
 */
void
component_client::checkNasdQueues (list<event *> *eventList, simtime_t currTime)
{
    ASSERT (eventList != NULL);
    
    for (PQsHashIter pQsHashIter = pQsHash.begin (); 
	 pQsHashIter != pQsHash.end (); 
	 pQsHashIter++) {
	
	queue <event *> *q = pQsHashIter->second;
	sim_component_id_t nasdId = pQsHashIter->first;
	
	ASSERT (creditHash.find (nasdId) != creditHash.end ());
	CreditStatus * cS = creditHash [nasdId];
	ASSERT (cS != NULL);
	int avail = cS->total - cS->inUse;
	
	while (avail > 0 && !q->empty ()) {
	    event * evt = q->front (); 
	    q->pop ();
	    
	    /*
	      Removed on 04-30-2000 by ozaki
	      
	      if (currTime > evt->get_start_time()) {
	      evt->set_start_time (currTime); // adjust start time
	      }
	    */
	    
	    if (currTime > evt->get_injection_time ()) {
	      evt->set_start_time (currTime);
	      evt->set_injection_time (currTime);
	    }
	    
	    ASSERT(evt->get_start_time() >= evt->get_injection_time());
	    eventList->push_front (evt);
	    avail --;
	    cS->inUse++;
	}
    }
}

void 
component_client::checkPartialReadRequests (
					    simtime_t startTime,
					    simtime_t finishTime,
					    param_read *p
					    )
{
  ASSERT (p != NULL);
  
  if (p->get_partial()) {
    
    if (partialReadHash.find (p->get_request_id ()) == partialReadHash.end ()) {
      // request not completed. Should be put in hash table
      partialReadHash [p->get_request_id ()] = p;
      return;
    }
    
    param_read *otherP = partialReadHash [p->get_request_id ()];
    ASSERT (otherP != NULL);
      
    otherP->inc_length (p->get_length ());
    
    if (otherP->get_length () >= otherP->get_total_length ()) {
      partialReadHash.erase (otherP->get_request_id ());
      
      p->set_length (otherP->get_length ());
      
      // ASSUMPTION: injection time is accurate
      delete otherP;
    }
    else {
      delete p; return;
    }
  }
  
  incCredit (p->get_nasd_id ());
    
  updateStream (p->get_stream_id ()); //, p->get_nasd_id ());
  
  updateReadStats (startTime, finishTime, p->get_nasd_id (), p->get_length ());
  
  delete p;
}
   

void 
Stats::dumpStats (ostream & output) const
{
  output << "numReads                   = " << numReads << endl
	 << "Average read request time  = " << avgReadTime << " seconds" << endl
	 << "Average read rate          = " << avgReadRate << " bytes / second" << endl
	 << "numWrites                  = " << numWrites << endl
	 << "Average write request time = " << avgWriteTime << " seconds" << endl
	 << "Average write rate         = " << avgWriteRate << " bytes / second" << endl
	 << "____________________________" << endl;
}

void 
Stats::updateRead (simtime_t startTime, 
		    simtime_t finishTime, 
		    sim_len_t dataSize)
{
  simtime_t timeTaken = finishTime - startTime;
  
  avgReadRate = 
    ((avgReadRate * totalReadTime) + dataSize) / (totalReadTime + timeTaken);
  
  avgReadTime = ((avgReadTime * numReads) + timeTaken) / (numReads + 1);
  
  totalReadTime += timeTaken;
  numReads ++;
}

void 
Stats::updateWrite (simtime_t startTime, 
		     simtime_t finishTime, 
		     sim_len_t dataSize)
{
  simtime_t timeTaken = finishTime - startTime;
  
  avgWriteRate = 
    ((avgWriteRate * totalWriteTime) + dataSize) / (totalWriteTime + timeTaken);
  
  avgWriteTime = ((avgWriteTime * numWrites) + timeTaken) / (numWrites + 1);
  
  totalWriteTime += timeTaken;
  numWrites ++;
}

/**
 * destructor
 */
StreamStatus::~StreamStatus () 
{
  for (; !pQueue.empty (); pQueue.pop ())
    if (pQueue.front () != NULL) delete pQueue.front ();
}

event *
StreamStatus::dequeue (void)
{
    if (pQueue.empty ()) return NULL;
    event *evt = pQueue.front (); 
    pQueue.pop ();
    return evt;
}

sim_bandwidth_t
NasdStatus::getIntervalBandwidth (simtime_t currTime) const 
{
  simtime_t elapsedTime = currTime - startTime_;
  
  double bw;
  
  if (elapsedTime == 0) 
    bw = 0;
  else
    bw = bytesAccessed_ / (double)elapsedTime;
  
  return bw;
}












