//
// Class:
//  components
//
//  Description:
//   [see corresponding header file]
//

// $Id: nasdsim_components.C,v 1.5 2000/04/15 02:26:38 ozaki Exp $

#include "nasdsim_components.h"
#include "nasdsim_debug.h"

//
// components::components() [constructor]
//
// description:
//  construct an empty component object
//
// parameters:
//  none
//
components::components(void)
{
}

//
// components::~components() [destructor]
//
// description:
//  free any memory allocated
//
// parameters:
//  none
//
components::~components()
{
  nasds_list_iter_t nasds_iter;
  clients_list_iter_t clients_iter;
  
  // delete all of the nasds
  for (nasds_iter = nasds.begin(); nasds_iter != nasds.end(); nasds_iter++) {
    delete nasds_iter->second;
  }
  
  // delete all clients
  for (clients_iter = clients.begin(); clients_iter != clients.end(); clients_iter++)
    if (clients_iter->second != NULL) delete clients_iter->second;
}

//
// components::add_component_nasd()
//
// description:
//  Add a component_nasd object at the specified id
//
// parameters:
//  id - identifier for the new component_nasd object.
//  comp - component_nasd object to add 
//
// return values:
//  none
//
void
components::add_component_nasd(
			       sim_component_id_t id,
			       component_nasd *comp
			       )
{
  // make sure the nasds doesn't already exist
  ASSERT(nasds.find(id) == nasds.end());

  nasds[id] = comp;
}

//
// components::get_component_nasd()
//
// description:
//  retrieve a component_nasd object according to id
//
// parameters:
//  id - identifier for the requested component_nasd object
//
// return values:
//  pointer to the request component_nasd;
//
component_nasd *
components::get_component_nasd(
			       sim_component_id_t id
			       )
{
  // check if the nasd exists
  ASSERT(nasds.find(id) != nasds.end());

  return nasds[id];
}

void
components::add_component_client (sim_component_id_t id, 
				  component_client *comp)
{
  ASSERT (clients.find (id) == clients.end());

  clients [id] = comp;
}

component_client *
components::get_component_client (sim_component_id_t id)
{
  ASSERT (clients.find(id) != clients.end());
  
  return clients [id];
}

//
// components::dump_stats()
//
// description:
//  dump stats for all components
//
// parameters:
//  output - the destination output stream for the statistics
//
// return values:
//  none
//
void
components::dump_stats(
		       ostream &output
		       ) const
{
  nasds_list_iter_t nasds_iter;

  for (nasds_iter = nasds.begin();
       nasds_iter != nasds.end(); nasds_iter++) {
    nasds_iter->second->dump_stats(output);
    output << endl;
  }
  
  clients_list_iter_t clients_iter;

  for (clients_iter = clients.begin();
       clients_iter != clients.end(); clients_iter++) {
    clients_iter->second->dump_stats (output);
    output << endl;
  }
}
