// $Id: nasdsim_event_READ.C,v 1.8 2000/04/24 23:01:54 ozaki Exp $

#include "nasdsim_event_READ.h"
#include "nasdsim_event_nasd.h"

//-------------------------------------------------------------------------------------
// event_READ_client_read
//-------------------------------------------------------------------------------------

events_t *
event_READ_client_read::simulate (components &comp)
{
  event_READ_switch_recv_from_client *newEvent;
  component_link *clientLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  clientLink = (comp.get_component_client (parameter->get_client_id ()))->getLink ();
  
  requestSize = clientLink->getLinkHdrSize ();
  
  clientLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_READ_switch_recv_from_client (completionTime, 
						     injection_time_,
						     parameter);
  events = new events_t;
  events->push(newEvent);
  
  return events;
}

#if defined(DEBUG)
void
event_READ_client_read::print(
			      ostream &output
			      ) const
    {
	output << "event_READ_client_read: { start = " << start_time_;
	parameter->print (output);
	output << " }" << endl;
    }
#endif // defined(DEBUG)
 
//-------------------------------------------------------------------------------------
// event_READ_switch_recv_from_client
//-------------------------------------------------------------------------------------

events_t *
event_READ_switch_recv_from_client::simulate (components &comp)
{
  event_READ_switch_send_to_nasd *newEvent;
  simtime_t completionTime;
 events_t *events;
  completionTime = start_time_ + NASDSIM_SWITCH_TIME;
  
  newEvent = new event_READ_switch_send_to_nasd (completionTime, 
						 injection_time_,
						 parameter);
  events = new events_t;
  events->push(newEvent);
  
  return events;
}


#if defined(DEBUG)
void
event_READ_switch_recv_from_client::print(
					  ostream &output
					  ) const
{
    output << "event_READ_swtich_recv_from_client: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_READ_switch_send_to_nasd
//-------------------------------------------------------------------------------------

events_t *
event_READ_switch_send_to_nasd::simulate (components &comp)
{
  event_READ_nasd_recv_from_switch *newEvent;
  component_link *nasdLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  nasdLink = (comp.get_component_nasd (parameter->get_nasd_id ()))->getLink ();
  
  requestSize = nasdLink->getLinkHdrSize ();
  
  nasdLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_READ_nasd_recv_from_switch (completionTime, 
						   injection_time_,
						   parameter);
  events = new events_t;
  events->push(newEvent);
  
  return events;
}
#if defined(DEBUG)
void
event_READ_switch_send_to_nasd::print(
				      ostream &output
				      ) const
{
  output << "event_READ_switch_send_to_nasd: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl;
}
#endif // defined(DEBUG)
//-------------------------------------------------------------------------------------
// event_READ_nasd_recv_from_switch
//-------------------------------------------------------------------------------------

events_t *
event_READ_nasd_recv_from_switch::simulate (components &comp)
{
  event_nasd_read *newEvent;
  events_t *events;
  
  newEvent = new event_nasd_read (this, start_time_, parameter);
  
  events = new events_t;
  events->push(newEvent);
  
  return events;
}

#if defined(DEBUG)
void
event_READ_nasd_recv_from_switch::print(
					ostream &output
					) const
{
    output << "event_READ_nasd_recv_from_switch: { start = " << start_time_;
    parameter->print (output);
    output << " }" << endl;
}
#endif // defined(DEBUG)

/******************************
 *                            *
 * Brian's code comes in here **********************************************************
 *                            *
 ******************************/

//-------------------------------------------------------------------------------------
// event_READ_nasd_send_to_switch
//-------------------------------------------------------------------------------------
events_t *
event_READ_nasd_send_to_switch::simulate (components &comp)
{
    event_READ_switch_recv_from_nasd *newEvent;
    component_link *nasdLink;
    simtime_t completionTime;
    sim_len_t requestSize;
    events_t *events;
     
    nasdLink = (comp.get_component_nasd (parameter->get_nasd_id ()))->getLink ();
    
    requestSize = nasdLink->getLinkHdrSize () + parameter->get_length ();
    
    nasdLink->queue_request (requestSize, start_time_, completionTime);
    
    newEvent = new event_READ_switch_recv_from_nasd (completionTime, 
						     injection_time_,
						     parameter);
    events = new events_t;
    events->push(newEvent);
    
    return events;
}

#if defined(DEBUG)
void
event_READ_nasd_send_to_switch::print(
				      ostream &output
				      ) const
{
  output << "event_READ_nasd_send_to_switch: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl;
}
#endif // defined(DEBUG)
		
//-------------------------------------------------------------------------------------
// event_READ_switch_recv_from_nasd
//-------------------------------------------------------------------------------------

events_t *
event_READ_switch_recv_from_nasd::simulate (components &comp)
{
  event_READ_switch_send_to_client *newEvent;
  simtime_t completionTime;
  events_t *events;
   
  // xxx assuming that it takes constant time inside a switch
  completionTime = start_time_ + NASDSIM_SWITCH_TIME;
  
  newEvent = new event_READ_switch_send_to_client (completionTime, 
						   injection_time_,
						   parameter);
  
  events = new events_t;
  events->push(newEvent);
  return events;
}

#if defined(DEBUG)
void
event_READ_switch_recv_from_nasd::print(
					ostream &output
					) const
{
  output << "event_READ_switch_recv_from_nasd: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_READ_switch_send_to_client
//-------------------------------------------------------------------------------------

events_t *
event_READ_switch_send_to_client::simulate (components &comp)
{
  event_READ_client_recv_from_switch *newEvent;
  component_link *clientLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  clientLink = (comp.get_component_client (parameter->get_client_id ()))->getLink ();
  
  requestSize = clientLink->getLinkHdrSize () + parameter->get_length ();
  
  clientLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_READ_client_recv_from_switch (completionTime, 
						     injection_time_,
						     parameter);
  events = new events_t;
  events->push(newEvent);
  return events;
}

#if defined(DEBUG)
void
event_READ_switch_send_to_client::print(
					ostream &output
					) const
{
  output << "event_READ_switch_send_to_client: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl;
}
#endif // defined(DEBUG)
//-------------------------------------------------------------------------------------
// event_READ_client_recv_from_switch
//-------------------------------------------------------------------------------------

events_t *
event_READ_client_recv_from_switch::simulate (components &comp)
{
  event_empty *newEvent;
  component_client *client;
  events_t *events;
  
  // For empty event
  param *p = new param (parameter->get_client_id (), parameter->get_nasd_id ());
  
  // Get client
  client = (comp.get_component_client (parameter->get_client_id ()));
  
  client->checkPartialReadRequests (injection_time_, start_time_, parameter);
  
  // Generate an empty event
  newEvent = new event_empty (dynamic_cast<event *>(this), start_time_, p);
  events = new events_t;
  events->push(newEvent);
  return events;
}

#if defined(DEBUG)
void
event_READ_client_recv_from_switch::print(
					ostream &output
					) const
{
  output << "event_READ_client_recv_from_switch: { start = " << start_time_;
  parameter->print (output);
  output << " }" << endl
	 << ">>> Generating an empty event >>>" << endl;
}
#endif // defined(DEBUG)
