// $Id: nasdsim_event_WRITE.C,v 1.9 2000/05/07 20:40:24 ozaki Exp $

#include "nasdsim_event_WRITE.h"
#include "nasdsim_event_nasd.h"
//-------------------------------------------------------------------------------------
// event_WRITE_client_write
//-------------------------------------------------------------------------------------

event_WRITE_client_write::event_WRITE_client_write (simtime_t stT,
						    simtime_t inT,
						    sim_component_id_t clId,
						    sim_stream_id_t strId,
						    sim_component_id_t nId,
						    sim_object_t obj,
						    sim_offset_t off,
						    sim_len_t len) 
  : event (stT, inT)
{
  clientId = clId;
  streamId = strId;
  nasdId = nId;
  object = obj;
  offset = off;
  length = len;
}

event_WRITE_client_write::~event_WRITE_client_write ()
{
}

events_t *
event_WRITE_client_write::simulate (components &comp)
{
  event_WRITE_switch_recv_from_client *newEvent;
  component_link *clientLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  clientLink = (comp.get_component_client (clientId))->getLink ();
  
  requestSize = clientLink->getLinkHdrSize () + length;
    
  clientLink->queue_request (requestSize, start_time_, completionTime);
  newEvent = new event_WRITE_switch_recv_from_client (completionTime, 
						      injection_time_,
						      clientId,
						      streamId,
						      nasdId,
						      object,
						      offset,
						      length);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_client_write::print(
			      ostream &output
			      ) const
{
  output << "event_WRITE_client_write: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "object = " << object << " offset = " << offset << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_WRITE_switch_recv_from_client
//-------------------------------------------------------------------------------------

event_WRITE_switch_recv_from_client::event_WRITE_switch_recv_from_client (simtime_t stT,
									  simtime_t inT,
									  sim_component_id_t clId,
									  sim_stream_id_t strId,
									  sim_component_id_t nId,
									  sim_object_t obj,
									  sim_offset_t off,
									  sim_len_t len)
  : event (stT, inT)
{
  clientId = clId;
  streamId = strId;
  nasdId = nId;
  object = obj;
  offset = off;
  length = len;
}

event_WRITE_switch_recv_from_client::~event_WRITE_switch_recv_from_client ()
{
}

events_t *
event_WRITE_switch_recv_from_client::simulate (components &comp)
{
  event_WRITE_switch_send_to_nasd *newEvent;
  simtime_t completionTime;
  events_t *events;
  
  completionTime = start_time_ + NASDSIM_SWITCH_TIME;
  
  newEvent = new event_WRITE_switch_send_to_nasd (completionTime, 
						  injection_time_,
						  clientId,
						  streamId,
						  nasdId,
						  object,
						  offset,
						  length);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_switch_recv_from_client::print(
					  ostream &output
					  ) const
{
  output << "event_WRITE_swtich_recv_from_client: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "object = " << object << " offset = " << offset << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_WRITE_switch_send_to_nasd
//-------------------------------------------------------------------------------------

event_WRITE_switch_send_to_nasd::event_WRITE_switch_send_to_nasd (simtime_t stT,
								  simtime_t inT,
								  sim_component_id_t clId,
								  sim_stream_id_t strId,
								  sim_component_id_t nId,
								  sim_object_t obj,
								  sim_offset_t off,
								  sim_len_t len)
  : event (stT, inT)
{
  clientId = clId;
  streamId = strId;
  nasdId = nId;
  object = obj;
  offset = off;
  length = len;
}

event_WRITE_switch_send_to_nasd::~event_WRITE_switch_send_to_nasd ()
{
}

events_t *
event_WRITE_switch_send_to_nasd::simulate (components &comp)
{
  event_WRITE_nasd_recv_from_switch *newEvent;
  component_link *nasdLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  nasdLink = (comp.get_component_nasd (nasdId))->getLink ();
  
  requestSize = nasdLink->getLinkHdrSize () + length;
  
  nasdLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_WRITE_nasd_recv_from_switch (completionTime, 
						    injection_time_,
						    clientId,
						    streamId,
						    nasdId,
						    object,
						    offset,
						    length);
  events = new events_t;
  events->push(newEvent);

  return events;
}
#if defined(DEBUG)
void
event_WRITE_switch_send_to_nasd::print(
				      ostream &output
				      ) const
{
  output << "event_WRITE_switch_send_to_nasd: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "object = " << object << " offset = " << offset << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)
//-------------------------------------------------------------------------------------
// event_WRITE_nasd_recv_from_switch
//-------------------------------------------------------------------------------------

event_WRITE_nasd_recv_from_switch::event_WRITE_nasd_recv_from_switch (simtime_t stT,
								      simtime_t inT,
								      sim_component_id_t clId,
								      sim_stream_id_t strId,
								      sim_component_id_t nId,
								      sim_object_t obj,
								      sim_offset_t off,
								      sim_len_t len)
  : event (stT, inT)
{
  clientId = clId;
  streamId = strId;
  nasdId = nId;
  object = obj;
  offset = off;
  length = len;
}

event_WRITE_nasd_recv_from_switch::~event_WRITE_nasd_recv_from_switch ()
{
}

events_t *
event_WRITE_nasd_recv_from_switch::simulate (components &comp)
{
  event_nasd_write *newEvent;
  param_write *p;
  events_t *events;
  
  p = new param_write (object, offset, length, clientId, streamId, nasdId);
  
  newEvent = new event_nasd_write (this, start_time_, p);
  events = new events_t;
  events->push(newEvent);
  
  return events;
}
#if defined(DEBUG)
void
event_WRITE_nasd_recv_from_switch::print(
					 ostream &output
					 ) const
{
  output << "event_WRITE_nasd_recv_from_switch: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
    	 << "object = " << object << " offset = " << offset << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)

/******************************
 *                            *
 * Brian's code comes in here **********************************************************
 *                            *
 ******************************/

//-------------------------------------------------------------------------------------
// event_WRITE_nasd_send_to_switch
//-------------------------------------------------------------------------------------
event_WRITE_nasd_send_to_switch::event_WRITE_nasd_send_to_switch (event * evt,
								  simtime_t stT,
								  param_write *p)
  : event (evt, stT)
{
  nasdId = p->get_nasd_id ();
  clientId = p->get_client_id ();
  streamId = p->get_stream_id (); 
  length = p->get_length ();
}

event_WRITE_nasd_send_to_switch::~event_WRITE_nasd_send_to_switch ()
{
}

events_t *
event_WRITE_nasd_send_to_switch::simulate (components &comp)
{
  event_WRITE_switch_recv_from_nasd *newEvent;
  component_link *nasdLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  nasdLink = (comp.get_component_nasd (nasdId))->getLink ();
  
  requestSize = nasdLink->getLinkHdrSize ();
  
  nasdLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_WRITE_switch_recv_from_nasd (completionTime, 
						    injection_time_,
						    clientId,
						    streamId, 
						    nasdId,
						    length);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_nasd_send_to_switch::print(
				      ostream &output
				      ) const
{
  output << "event_WRITE_nasd_send_to_switch: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
      	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)
		
//-------------------------------------------------------------------------------------
// event_WRITE_switch_recv_from_nasd
//-------------------------------------------------------------------------------------

event_WRITE_switch_recv_from_nasd::event_WRITE_switch_recv_from_nasd (simtime_t stT,
								      simtime_t inT,
								      sim_component_id_t clId,
								      sim_stream_id_t strId,
								      sim_component_id_t nId,
								      sim_len_t len)
  : event (stT, inT)
{
  clientId = clId;
  nasdId = nId;
  streamId = strId;
  length = len;
}

event_WRITE_switch_recv_from_nasd::~event_WRITE_switch_recv_from_nasd ()
{
}

events_t *
event_WRITE_switch_recv_from_nasd::simulate (components &comp)
{
  event_WRITE_switch_send_to_client *newEvent;
  simtime_t completionTime;
  events_t *events;
  
  // xxx assuming that it takes constant time inside a switch
  completionTime = start_time_ + NASDSIM_SWITCH_TIME;
  
  newEvent = new event_WRITE_switch_send_to_client (completionTime, 
						    injection_time_,
						    clientId,
						    streamId,
						    nasdId,
						    length);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_switch_recv_from_nasd::print(
					 ostream &output
					 ) const
{
  output << "event_WRITE_switch_recv_from_nasd: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
      	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_WRITE_switch_send_to_client
//-------------------------------------------------------------------------------------

event_WRITE_switch_send_to_client::event_WRITE_switch_send_to_client (simtime_t stT,
								      simtime_t inT,
								      sim_component_id_t clId,
								      sim_stream_id_t strId,
								      sim_component_id_t nId,
								      sim_len_t len)
  : event (stT, inT)
{
  nasdId = nId;
  clientId = clId;
  streamId = strId;
  length = len;
}

event_WRITE_switch_send_to_client::~event_WRITE_switch_send_to_client ()
{
}

events_t *
event_WRITE_switch_send_to_client::simulate (components &comp)
{
  event_WRITE_client_recv_from_switch *newEvent;
  component_link *clientLink;
  simtime_t completionTime;
  sim_len_t requestSize;
  events_t *events;
  
  clientLink = (comp.get_component_client (clientId))->getLink ();
  
  requestSize = clientLink->getLinkHdrSize ();
  
  clientLink->queue_request (requestSize, start_time_, completionTime);
  
  newEvent = new event_WRITE_client_recv_from_switch (completionTime, 
						      injection_time_,
						      clientId,
						      streamId,
						      nasdId,
						      length);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_switch_send_to_client::print(
					 ostream &output
					 ) const
{
  output << "event_WRITE_switch_send_to_client: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "length = " << length << " }" << endl;
}
#endif // defined(DEBUG)

//-------------------------------------------------------------------------------------
// event_WRITE_client_recv_from_switch
//-------------------------------------------------------------------------------------

event_WRITE_client_recv_from_switch::event_WRITE_client_recv_from_switch (simtime_t stT,
									  simtime_t inT,
									  sim_component_id_t clId,
									  sim_stream_id_t strId,
									  sim_component_id_t nId,
									  sim_len_t len)
    : event (stT, inT)
{
  nasdId = nId;
  clientId = clId;
  streamId = strId;
  length = len;
}

event_WRITE_client_recv_from_switch::~event_WRITE_client_recv_from_switch ()
{
}

events_t *
event_WRITE_client_recv_from_switch::simulate (components &comp)
{
  event_empty *newEvent;
  component_client *client;
  events_t *events;
  
  client = (comp.get_component_client (clientId));
  
  // increase credit of given nasd
  client->incCredit (nasdId);
  
  // update stream info
  client->updateStream (streamId); //, nasdId);
  
  client->updateWriteStats (injection_time_, start_time_, nasdId, length);
  
  // Generate an empty event
  param *p = new param (clientId, nasdId); 
  newEvent = new event_empty (dynamic_cast<event *>(this),
			      start_time_,
			      p);
  events = new events_t;
  events->push(newEvent);

  return events;
}

#if defined(DEBUG)
void
event_WRITE_client_recv_from_switch::print(
					   ostream &output
					   ) const
{
  output << "event_WRITE_client_recv_from_switch: { start = " << start_time_ << " "
	 << "clientId = " << clientId << " "
	 << "streamId = " << streamId << " "
	 << "nasdId = " << nasdId << " "
	 << "length = " << length << " }" << endl
	 << ">>> Generating an empty event >>>" << endl;
}
#endif // defined(DEBUG)
