// $Id: nasdsim_event_WRITE.h,v 1.6 2000/05/07 03:08:51 ozaki Exp $

#if !defined(NASDSIM_EVENT_WRITE_H)
#define NASDSIM_EVENT_WRITE_H

#include "nasdsim_event.h"
#include "nasdsim_types.h"
#include "nasdsim_components.h"
#include "nasdsim_param.h"

class event_WRITE_client_write : public event
{
 private:
  
  sim_component_id_t clientId;
  sim_stream_id_t streamId;
  sim_component_id_t nasdId;
  sim_object_t object;
  sim_offset_t offset;
  sim_len_t length;
  
 public:
  
  event_WRITE_client_write (simtime_t stT, simtime_t inT, sim_component_id_t clId, 
			    sim_stream_id_t strId,
			    sim_component_id_t nId, sim_object_t obj, sim_offset_t off,
			    sim_len_t len);
  ~event_WRITE_client_write();
  events_t *simulate(components &comp);
  sim_component_id_t get_nasd_id (void) { return nasdId; }
  sim_stream_id_t get_stream_id (void) { return streamId; }
  
#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_switch_recv_from_client : public event 
{
 private:
  
  sim_component_id_t clientId;
  sim_stream_id_t streamId;
  sim_component_id_t nasdId;
  sim_object_t object;
  sim_offset_t offset;
  sim_len_t length;
  
 public:
  
  event_WRITE_switch_recv_from_client (simtime_t stT, simtime_t inT, 
				       sim_component_id_t clId,
				       sim_stream_id_t strId,
				       sim_component_id_t nId,
				       sim_object_t obj, 
				       sim_offset_t off, 
				       sim_len_t len);
  ~event_WRITE_switch_recv_from_client ();
  events_t *simulate (components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_switch_send_to_nasd : public event 
{
 private:
  
  sim_component_id_t clientId;
  sim_stream_id_t streamId;
  sim_component_id_t nasdId;
  sim_object_t object;
  sim_offset_t offset;
  sim_len_t length;
  
 public:
  
  event_WRITE_switch_send_to_nasd (simtime_t stT, simtime_t inT, sim_component_id_t clId,
				   sim_stream_id_t strId,
				   sim_component_id_t nId, sim_object_t obj,
				   sim_offset_t off,
				   sim_len_t len);
  ~event_WRITE_switch_send_to_nasd ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_nasd_recv_from_switch : public event 
{
 private:
  sim_component_id_t clientId;
  sim_component_id_t nasdId;
  sim_object_t object;
  sim_offset_t offset;
  sim_len_t length;
  sim_stream_id_t streamId;
  
 public:
  
  event_WRITE_nasd_recv_from_switch (simtime_t stT, simtime_t inT, sim_component_id_t clId,
				     sim_stream_id_t strId,
				     sim_component_id_t nId, sim_object_t obj, 
				     sim_offset_t off, sim_len_t len);
  ~event_WRITE_nasd_recv_from_switch ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

/******************************
 *                            *
 * Brian's code comes in here **********************************************************
 *                            *
 ******************************/

class event_WRITE_nasd_send_to_switch : public event
{
 private:
  
  sim_component_id_t nasdId;
  sim_stream_id_t streamId;
  sim_component_id_t clientId;
  sim_len_t length;
  
 public:
  
  event_WRITE_nasd_send_to_switch (event *, simtime_t, param_write *);
  ~event_WRITE_nasd_send_to_switch ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_switch_recv_from_nasd : public event
{
 private:
  sim_stream_id_t streamId;
  sim_component_id_t clientId, nasdId;
  sim_len_t length;
  
 public:
  
  event_WRITE_switch_recv_from_nasd (simtime_t stT, simtime_t inT, sim_component_id_t clId,
				     sim_stream_id_t strId,
				    sim_component_id_t nId, sim_len_t len);
  ~event_WRITE_switch_recv_from_nasd ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_switch_send_to_client : public event
{
 private:
  
  sim_component_id_t clientId, nasdId;
  sim_stream_id_t streamId;
  sim_len_t length;
  
 public:
  
  event_WRITE_switch_send_to_client (simtime_t stT, simtime_t inT, sim_component_id_t clId, 
				     sim_stream_id_t strId,
				    sim_component_id_t nId, sim_len_t len);
  ~event_WRITE_switch_send_to_client ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

class event_WRITE_client_recv_from_switch : public event
{
 private:
  
  sim_component_id_t clientId, nasdId;
  sim_stream_id_t streamId;
  sim_len_t length;
  
 public:
  
  event_WRITE_client_recv_from_switch (simtime_t stT, simtime_t inT, sim_component_id_t clId,
				       sim_stream_id_t strId,
				       sim_component_id_t nId, sim_len_t len);
  ~event_WRITE_client_recv_from_switch ();
  events_t *simulate(components &comp);

#if defined (DEBUG)
  void print(ostream &o)const;
#endif
};

#endif // #define NASDSIM_EVENT_WRITE_H




