//
// Description:
//  This file contains class interfaces for all of the NASD events.
//  event_nasd, event_nasd_read, event_nasd_read_complete, event_nasd_write,
//  and event_nasd_write_complete classes are in this file. All of these
//  class are in the event class hierarchy.
//

// $Id: nasdsim_event_nasd.h,v 1.7 2000/04/30 03:43:03 bforney Exp $

#if !defined(NASDSIM_EVENT_NASD_H)
#define NASDSIM_EVENT_NASD_H

#include <iostream>
#include "nasdsim_event.h"
#include "nasdsim_types.h"
#include "nasdsim_components.h"
#include "nasdsim_param.h"

class event_nasd : public event {
 public:
  // constructor and destructor
  event_nasd(const event *copy_event, simtime_t time);
  ~event_nasd();

  // generators
  virtual events_t *simulate(components &comp) = 0; // abstract

 protected:
};


class event_nasd_read : public event_nasd {
 public:
  // constructor and destructor
  event_nasd_read(const event *copy_event, simtime_t start_time,
		  param_read *read_param);
  ~event_nasd_read();

  // accessors
  param_read *get_param(void) const { return parameter_; }

  // generators
  events_t *simulate(components &comp);

#if defined(DEBUG)
  void print(ostream &output) const;
#endif // defined(DEBUG)

 protected:
  param_read *parameter_;
};

class event_nasd_write : public event_nasd {
 public:
  // constructor and destructor
  event_nasd_write(const event *copy_event, simtime_t start_time,
		   param_write *write_param);
  ~event_nasd_write();

  // accessors
  param_write *get_param(void) const { return parameter_; }

  // generators
  events_t *simulate(components &comp);

#if defined(DEBUG)
  void print(ostream &output) const;
#endif // defined(DEBUG)

 protected:
  param_write *parameter_;
};

#if defined(NASD_REMOTE_CACHING)
class event_nasd_N2N : public event_nasd {
 public:
  // constructor and destructor
  event_nasd_N2N(const event *copy_event, simtime_t start_time,
		 param_nasd_2_nasd *paramter); // xxx fix the type
  ~event_nasd_N2N();

  // accessors
  param_nasd_2_nasd *get_param(void) const { return parameter_; }

  // generators
  events_t *simulate(components &comp);

#if defined(DEBUG)
  void print(ostream &output) const;
#endif // defined(DEBUG)

 private:
  void global_cache_add(components& comp);
  events_t *global_cache_read(components& comp);
  void global_cache_invalidate(components& comp);
  events_t *generate_local_reads(components& comp);

 private:
  param_nasd_2_nasd *parameter_;
};
#endif // defined(NASD_REMOTE_CACHING)

#endif // #define NASDSIM_EVENT_NASD_H


