//
// Classes:
//  param_read, param_write
//
// Description:
//  The vast majority (and I think all) NASD operations require some kind of
//  parameters. Rather than pass the parameters as basic C++ types, which
//  is cumbersome and less than easy to extend, when a new event in the
//  request event chain is created, param objects are passed. This is much
//  easier to handle as the simulator changes to meet new needs.
//

// $Id: nasdsim_param.C,v 1.3 2000/04/30 03:43:04 bforney Exp $

#include "nasdsim_param.h"

// *****
//
// param_read
//

void
param_read::print(
		  ostream &output
		  ) const
{
#if defined(DEBUG)
    output << " param_read: {"
	   << "client_id=" << client_id 
	   << " nasd_id=" << nasd_id 
	   << " stream_id=" << stream_id
	   << " request_id=" << request_id_
	   << " object=" << object_ 
	   << " offset=" << offset_
	   << " length=" << length_
	   << " partial=" << partial_
	   << "}";
#endif // defined(DEBUG)
}


// *****
//
// param_write
//

void
param_write::print(
		   ostream &output
                  ) const
{
#if defined(DEBUG)
  output << " param_write: { object = " << object_ << ", offset =  ";
  output << offset_ << ", length = " << length_ << " }";
#endif // defined(DEBUG)
}

void
param_parent_eviction_request::print(
				      ostream &output
				      ) const
{
#if defined(DEBUG)
    output << " param_parent_eviction_request: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_;
    
    if (block_list_ != NULL) 
	{
	    output << " block List -->" << endl;
	    
	    block_list_t::iterator blockListIter;
	    
	    for (blockListIter = block_list_->begin (); 
		 blockListIter != block_list_->end (); 
		 blockListIter ++)  {
		
	      output << "[object_id = " << blockListIter->object_id_ 
		     << ", block_id = " << blockListIter->block_id_ << "]";
		output << endl;
	    }
	}
    
    output << "}" << endl;
#endif // defined(DEBUG)
}

void
param_parent_eviction_reply::print(
				    ostream &output
				    ) const
{
#if defined(DEBUG)
    output << " param_parent_eviction_reply: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_
	   << " reply = " << reply_
	   << "}" << endl;
#endif // defined(DEBUG)
}

void
param_read_forward_request::print(
				  ostream &output
				  ) const
{
#if defined(DEBUG)
    output << " param_read_forward_request: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_ << endl;
    
    if (parameter_ != NULL) {
	parameter_->print (output); output << endl;
    }
    
    if (block_list_ != NULL) 
	{
	    output << "block List -->" << endl;
	    
	    block_list_t::iterator blockListIter;
	    
	    for (blockListIter = block_list_->begin (); 
		 blockListIter != block_list_->end (); 
		 blockListIter ++)  {
		
	      output << "[object_id = " << blockListIter->object_id_ 
		     << ", block_id = " << blockListIter->block_id_ << "]";

		output << endl;
	    }
	}
    
    output << "}" << endl;
#endif // defined(DEBUG)
}

#if 0
void
param_remote_eviction_request::print(
				     ostream &output
				     ) const
{
#if defined(DEBUG)
    output << " param_remote_eviction_request: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_ << endl;
    
    if (block_list_ != NULL) 
	{
	    output << "block List -->" << endl;
	    
	    block_list_t::iterator blockListIter;
	    
	    for (blockListIter = block_list_->begin (); 
		 blockListIter != block_list_->end (); 
		 blockListIter ++)  {
		
	      output << "[object_id = " << blockListIter->object_id_ 
		     << ", block_id = " << blockListIter->block_id_ << "]";
		output << endl;
	    }
	}
    
    output << "}" << endl;
#endif // defined(DEBUG)
}

void
param_remote_eviction_reply::print(
				    ostream &output
				    ) const
{
#if defined(DEBUG)
    output << " param_remote_eviction_reply: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_
	   << " reply = " << reply_
	   << "}" << endl;
#endif // defined(DEBUG)
}
#endif

void
param_parent_invalidate_request::print(
				       ostream &output
				       ) const
{
#if defined(DEBUG)
    output << " param_parent_invalidate_request: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_ << endl;
    
    if (block_list_ != NULL) 
	{
	    output << "block List -->" << endl;
	    
	    block_list_t::iterator blockListIter;
	    
	    for (blockListIter = block_list_->begin (); 
		 blockListIter != block_list_->end (); 
		 blockListIter ++)  {
		
	      output << "[object_id = " << blockListIter->object_id_ 
		     << ", block_id = " << blockListIter->block_id_ << "]";
		output << endl;
	    }
	}
    
    output << "}" << endl;
#endif // defined(DEBUG)
}

void
param_returned_read_request::print(
				       ostream &output
				       ) const
{
#if defined(DEBUG)
    output << " param_returned_read_request: {"
	   << "source_nasd = " << source_nasd_
	   << " destination_nasd = " << destination_nasd_
	   << " length = " << length_ << endl;
    
    if (block_list_ != NULL) 
	{
	    output << "block List -->" << endl;
	    
	    block_list_t::iterator blockListIter;
	    
	    for (blockListIter = block_list_->begin (); 
		 blockListIter != block_list_->end (); 
		 blockListIter ++)  {
		
	      output << "[object_id = " << blockListIter->object_id_ 
		     << ", block_id = " << blockListIter->block_id_ << "]";
		output << endl;
	    }
	}
    
    output << "}" << endl;
#endif // defined(DEBUG)
}
